
# q-expansion of newform 1617.2.a.bb, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1617
weight = 2
poly_data = [-10, 11, 14, -8, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0], 1], [[-4, 0, 1, 0, 0], 1], [[0, -6, 0, 1, 0], 1], [[5, 0, -7, 0, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 1, 0, 0, 0], [1, 0, 0, 0, 0], [1, 0, 0, 0, -1], [0, 0, 0, 0, 0], [1, 0, 0, 0, 0], [-2, 1, -1, 1, 1], [0, -1, 0, 0, 0], [0, 0, -1, 1, 1], [3, 0, 1, 0, 1], [0, 0, 0, -1, 2], [-1, 0, 1, 0, 0], [4, -2, 0, 0, -1], [4, -1, 0, 1, 2], [2, -1, -1, -2, -1], [2, -4, -1, 0, 0], [3, -2, 0, -2, -1], [-3, 0, -1, 0, -1], [0, -3, 0, -3, 0], [3, -2, 1, -2, 0], [4, 2, -3, 0, 0], [-2, -3, 3, -1, 1], [2, 1, -3, -1, -3], [2, 2, 2, 0, -2], [0, 2, 0, 2, -2], [4, 2, -1, -4, -2], [6, 0, 2, 1, 0], [-2, 4, 1, 2, -3], [4, -1, 0, -1, 2], [0, -1, -1, -1, -3], [1, 4, -2, 2, 1], [4, 0, 3, 3, -1], [-6, 1, -2, -1, 0], [8, -6, 0, 2, 2], [-2, 1, 1, 0, -3], [-2, 1, 2, -2, -2], [-6, 5, 2, 0, 0], [-5, 0, 1, 6, 3], [5, -4, -2, 0, 3], [0, 2, -2, 0, 2], [4, 1, 0, -1, 4], [4, -6, 3, 2, 0], [0, 2, -3, 2, -1], [4, -6, 2, 2, -2], [4, -4, 1, -4, 1], [-6, -1, 0, 2, 0], [-2, 0, 2, 0, -2], [-2, -1, -4, 3, 0], [1, -2, 4, 0, 3], [6, -4, -2, 0, -4], [-6, -2, 1, -2, 5], [-6, 4, 0, -3, -2], [6, -1, 4, -3, -4], [-6, 8, 0, 0, 0], [-4, 6, -3, 2, 2], [-10, -4, 2, 0, 2], [18, -2, -2, -2, 0], [15, -4, -2, -4, -1], [0, -7, 0, 1, 2], [0, 6, -5, 0, -1], [0, 0, 4, 1, 4], [0, 7, 1, -3, -1], [-4, 2, -4, 1, 8], [-4, 1, 3, -1, 1], [5, -4, -3, -4, 3], [-18, 4, 2, 2, -1], [-5, 0, -2, 6, 3], [-6, 2, -1, 2, 9], [0, 2, 3, 2, -1], [-4, 1, 4, -3, -4], [-4, -5, 4, 3, 0], [-9, 6, -6, -4, -1], [16, -1, 2, -1, -2], [-5, 2, -5, -2, 2], [-20, 3, 1, 1, 7], [5, 8, -1, -2, -2], [3, 2, -5, -2, -1], [-4, 2, -4, 2, 6], [-2, 2, 2, -6, -5], [-11, 0, -4, -4, 7], [-4, -3, -1, 1, 3], [-4, 2, -3, 4, -8], [-3, -4, 0, -6, 4], [20, -3, -4, -3, -4], [6, -8, 4, 0, -1], [-14, -7, 0, -4, 0], [-1, 0, 5, 0, 7], [0, -4, 0, 0, 0], [-8, -3, -1, 5, -3], [18, -1, 2, -2, 0], [8, 4, 3, -2, 3], [-7, -4, 9, -2, -1], [2, 4, 8, 0, -4], [7, -6, 7, 0, 8], [-12, -1, 0, 5, 0], [5, 0, 1, 0, 0], [-10, -1, 2, -1, -8], [10, 12, -6, 2, 0], [4, 4, 6, 2, -2], [-4, -3, -3, -1, -3], [-12, -1, 5, 5, 9], [-6, 2, -2, 1, -2], [6, -5, 6, -4, -4], [4, -1, -4, -3, 4], [-4, 7, 2, -2, 4], [-6, 2, 1, 3, -3], [-4, -2, -5, 2, -7], [-18, 2, -4, 4, -2], [12, -2, 6, 1, -2], [6, 8, -2, 6, 4], [-10, 2, -3, -2, 1], [2, -7, 11, 3, 3], [2, 2, 2, 4, 0], [11, 6, -4, 0, -7], [17, -4, 3, -4, -4], [5, -12, -8, 2, 3], [-13, -8, 0, 2, 5], [8, -2, 1, -6, -1], [4, 0, 6, 0, 6], [-2, -2, -2, 8, 6], [-4, -12, 8, 2, 6], [-1, 8, 0, -4, -2], [22, 1, -5, -3, -3], [4, 7, -4, 0, -10], [4, 8, -5, -2, -4], [-10, 8, 1, 4, 7], [-8, -5, 4, -4, -4], [-2, 2, 1, 0, 4], [-8, 10, 3, 0, 2], [0, 14, -3, -4, 1], [-14, -3, -1, -1, -3], [2, 7, 1, -3, -5], [10, 10, -4, 0, -2], [6, -4, -5, -6, -5], [7, 0, 1, 2, 7], [10, 5, -2, -9, -6], [-2, -6, -3, 2, 5], [-10, -2, 0, 2, 8], [-2, -1, -4, 2, -12], [4, 4, -4, 2, 0], [-30, 5, 2, 3, 2], [-16, -5, -4, 3, -2], [-10, 5, -4, -1, -4], [21, -6, 0, 2, -1], [0, 0, 4, 0, -2], [-4, 2, 2, -2, -9], [-8, 4, 6, 2, 2], [2, 12, -5, 4, -7], [-4, 0, -4, -7, 0], [10, 12, -10, 0, -2], [18, 8, -4, 6, 2], [2, -15, 2, -1, -2], [20, -16, -2, -4, 2], [14, 10, 1, 4, -3], [28, -3, -6, 1, 4], [-3, -4, -15, 0, 2], [2, -6, 7, 0, 2], [-2, 4, 5, 1, 11], [-5, -4, 4, 14, 5], [26, 0, 3, 4, -7], [6, 0, 14, 1, -2], [4, -4, 3, 6, 6], [-24, 11, -2, 5, 0], [14, -12, 1, -5, 11], [4, 0, -4, 6, 12], [-3, 4, -6, 6, 7], [-9, 6, 9, 0, 3], [-30, -4, -7, -4, 3], [-14, 0, 1, 4, 13], [-14, 5, 5, -1, 3], [-18, -2, -2, 4, 2], [6, 2, 6, -4, -4], [-6, -2, 4, -10, -8], [26, -5, -2, 7, 8], [12, -8, 7, -4, -5], [8, -8, 3, -2, 15], [19, -8, -2, -6, -11], [4, -1, -1, -6, -5], [21, -10, 6, -8, 7], [4, -9, 8, 3, 6], [-8, -2, -3, -2, -5], [27, 6, 5, 0, -2], [-4, 13, -2, -7, -2], [-17, 2, -4, 2, -10], [24, -12, 4, -3, -12], [0, 16, 3, 10, -4], [12, 0, -4, -4, -4], [-10, 12, -4, 0, -4], [10, 4, 3, -2, 3], [-20, 12, 7, 2, -5], [-16, 15, -6, -1, 0], [-19, 4, 6, 10, 6], [0, -1, -2, 1, -4], [-2, 2, 6, -6, -6], [-10, -2, 8, -2, -14], [-12, 6, 0, 4, -8], [1, 2, -6, 2, -13], [-6, 7, -1, -7, 1], [-1, 8, 11, 2, -1], [-16, -9, 6, 9, 8], [-10, -8, -2, -2, 6], [-24, -3, 4, -1, -4], [14, 13, -9, 0, 3], [-2, 14, 0, -4, 8], [-6, 14, -4, 4, 4], [-46, -2, -1, 4, 0], [-19, -24, 6, 2, -1], [11, -12, 0, 0, 5], [20, -4, 4, -10, -6], [-20, 13, 10, -1, -8], [7, 4, -8, 10, 13], [-18, -12, 5, -4, -1], [10, 22, -10, 2, -8], [28, 12, 6, -4, 0], [-10, 2, 2, -4, 4], [20, -6, 6, -10, 10], [-6, 6, 6, 8, -5], [10, 1, -2, 5, -4], [6, -1, 8, -3, 8], [-4, 0, 0, 6, 8], [-8, 10, 4, -4, -2], [-6, 14, -5, 10, -7], [4, 3, -3, -6, 7], [-18, -10, 0, -4, -2], [27, 10, -8, 0, -7], [-4, -5, 12, -3, 0], [-16, -6, -4, 6, -6], [7, 6, 6, 0, 7], [21, 2, -1, -12, -7], [16, 12, -9, 7, 3], [-32, -18, 6, -4, 2], [-10, -4, -2, 12, 3], [-10, -4, 6, 3, 2], [16, -9, 5, 12, 3], [-42, 8, 2, 3, 0], [12, 10, 11, 2, -5], [-2, 1, -10, -1, -4], [1, -6, 6, -12, -2], [18, -9, 2, 1, -14], [-42, 12, -2, -4, -8], [-8, 8, 6, 10, 2], [17, 0, -13, 4, 5], [-18, 1, -3, 11, -7], [8, -2, -9, 4, -5], [-15, 10, -9, 0, 13], [2, 5, -12, 0, -10], [18, -11, 0, 3, 4], [-11, 8, -13, -2, 0], [18, -4, -3, 6, 4], [-24, 3, 0, 0, -18], [-8, -3, -4, -1, -4], [-24, -16, 1, 4, -1], [-36, 6, 0, -3, 8], [8, 4, 5, 0, -6], [-16, -2, 5, 4, -9], [38, -9, -2, -7, -2], [-14, 6, -10, 4, 10], [0, -14, -5, -4, 5], [8, -2, 1, -5, 7], [-4, -12, 4, 4, 6], [-22, -3, -8, 5, 2], [-14, -11, 17, -1, 9], [-22, 0, -6, 12, 8], [-38, 11, -1, 9, 5], [34, 0, 5, -6, 1], [-1, -6, 14, 8, 7], [-7, 8, 5, -6, 2], [1, -2, 4, 6, 5], [11, -2, 8, -14, -5], [2, -14, -6, -9, 10], [-20, -7, 4, -9, -6], [-14, -6, 2, -2, -5], [16, -12, 6, 8, 0], [12, -2, 1, -4, -17], [-4, -11, -1, 4, -13], [-16, -2, -4, 0, 14], [25, -2, 2, -2, -13], [10, -6, -1, -2, -4], [0, -10, 7, -6, 13], [10, -18, -5, 2, -1], [20, -9, 12, 7, -6], [8, -5, -6, -3, -8], [11, 6, 1, 4, -8], [-24, -20, 2, -2, 4], [-4, -4, -7, 4, -17], [6, -7, -8, -4, -4], [-17, 12, -5, 12, -11], [22, -2, -1, -8, -9], [-2, -14, 0, 5, 4], [21, 4, 13, 0, 2], [-14, 7, -12, -2, 4], [24, 18, -8, 4, -10], [-9, 14, 7, -6, -5], [-10, 17, -6, 7, -4], [18, -1, 4, -5, 12], [-14, 16, -3, 4, -9], [26, -13, 8, -7, -22], [31, -16, 1, -10, 12], [-22, -6, 10, -2, 4], [-4, -16, -6, -4, 6], [4, -12, -3, -7, -1], [-16, 19, 14, 1, -8], [10, 0, 8, 13, 2], [30, -12, 8, -15, -10], [-9, 18, -1, -6, -13], [21, 4, 9, 2, -8], [-6, -4, 7, -6, -5], [12, 2, -2, -10, -2], [-21, 22, 0, 12, 4], [19, -4, 21, 4, 1], [-24, -1, -7, -3, 11], [8, -9, 18, -7, -8], [0, -4, 6, -2, -22], [-36, -3, 4, 10, 4], [-17, 2, -8, 4, 17], [48, -13, -2, 3, 2], [-6, 7, 11, 11, 5], [-41, 0, -5, 0, -9], [-4, 1, 6, -5, 0], [-21, 16, -6, -4, 16], [20, 11, -14, -2, -6], [12, 9, 1, -4, -9], [11, 8, -10, 8, 8], [22, -3, 6, -8, 4], [23, 12, -19, 2, -4], [-4, 18, -12, -5, 2], [0, -26, -3, -6, 3], [1, -18, 19, 0, -2], [8, 0, 5, 8, 3], [-6, 12, -18, -4, -10], [-24, -5, 0, -7, -4], [-12, 3, -11, -5, -3], [6, 8, 8, -6, -16], [8, 1, 3, 0, -13], [12, 0, 2, 4, 8], [-22, 3, -5, 5, 13], [20, -4, 10, -6, -2], [10, 6, -11, 6, 12], [-32, 21, -4, 0, 8], [11, -6, 4, 6, 12], [0, 5, 7, 8, -9], [-12, 20, 1, 6, -9], [8, 6, -10, -8, 0], [8, -6, -14, -3, 8], [-14, 8, -9, 14, 15], [-31, 2, -8, 4, 9], [8, -13, 6, 17, 2], [-19, 14, -2, -12, -10], [16, 2, -8, 2, 12], [12, 6, -12, 2, -22], [-9, -18, -4, -8, 7], [-36, 11, -2, 4, 2], [1, -4, -3, 2, 17], [-9, -4, 0, 14, 1], [10, -20, 2, 7, -6], [4, -7, -7, -3, -1], [-24, 10, -2, 11, 12], [4, -12, -4, 8, 0], [-20, 10, -4, 2, 12], [30, -6, -2, 1, -4], [12, 8, -4, -2, 16], [-38, 7, 0, 1, -2], [10, -11, -10, -4, -2], [-14, 16, -2, -2, 24], [-24, -3, 8, -7, -2], [35, 2, -3, -4, 0], [23, 8, -7, -6, -9], [-6, -8, 0, 7, 8], [14, -15, -11, 7, 7], [-32, 21, 2, -5, -4], [9, 16, -15, 6, -5], [33, -16, -7, -18, -10], [0, 1, -4, -11, -14], [-14, -1, -8, 5, -14], [-38, 13, 15, 3, 3], [29, 6, -8, -6, 4], [-14, 6, -7, -2, 16], [50, 0, 2, 0, 6], [32, 5, 2, 11, 8], [-22, -3, -26, -1, 4], [16, -16, 3, -16, -7], [-9, 22, -12, 4, 3], [28, -6, 6, -4, 6], [12, 19, -6, -1, -14], [22, 7, -12, 2, 2], [-6, 16, -2, 13, 6], [20, -12, 10, -12, -10], [-11, 6, -9, -4, -6], [18, 2, -15, 6, 4], [-18, -7, 7, 12, 9], [20, -2, -3, 14, 4], [26, -12, 4, 2, -13], [20, 10, -4, 6, 10], [-14, 24, 5, -6, 2], [-17, 16, -15, -8, -2], [3, -16, 15, 2, -5], [-22, -1, -2, -1, 14], [-24, -7, 8, 2, 6], [-3, -12, 16, 0, -3], [-48, 4, -3, 8, -13], [-4, 17, -6, 8, -14], [-2, 18, 6, 18, 0], [26, 20, -2, 10, -8], [14, 15, 0, -5, 12], [3, 6, 12, -4, 5], [-8, -14, 6, -19, -4], [-14, -13, 15, -3, 9], [46, -8, -1, 0, 8], [-2, 9, -10, -6, -12], [37, -14, 7, -8, 2], [16, -10, 7, -16, -2], [-16, -2, 0, -6, 6], [8, 19, 10, -8, -6], [-32, 2, -5, -2, 18], [-4, -13, 14, -8, -18], [-2, -22, -1, -10, 9], [-18, 12, 16, 2, 8], [40, 10, -10, 8, 8], [-12, 10, -3, 18, 7], [-32, 6, -12, 20, -4], [-26, 23, -6, -5, -2], [12, -16, 6, 10, -8], [24, -15, -1, -7, 9], [30, -4, 7, 4, -12], [12, -23, 10, 7, 0], [-11, -24, 9, -10, 1], [51, -10, 12, -6, -1], [-24, 16, -6, 6, -26], [-10, -4, 4, 13, 10], [2, -10, 2, -4, 22], [24, -20, 0, 2, 4]]
